"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var lsp = __importStar(require("vscode-languageserver"));
// TODO: Move this to `@angular/language-service`.
var CompletionKind;
(function (CompletionKind) {
    CompletionKind["attribute"] = "attribute";
    CompletionKind["htmlAttribute"] = "html attribute";
    CompletionKind["property"] = "property";
    CompletionKind["component"] = "component";
    CompletionKind["element"] = "element";
    CompletionKind["key"] = "key";
    CompletionKind["method"] = "method";
    CompletionKind["pipe"] = "pipe";
    CompletionKind["type"] = "type";
    CompletionKind["reference"] = "reference";
    CompletionKind["variable"] = "variable";
    CompletionKind["entity"] = "entity";
})(CompletionKind || (CompletionKind = {}));
/**
 * Convert Angular's CompletionKind to LSP CompletionItemKind.
 * @param kind Angular's CompletionKind
 */
function ngCompletionKindToLspCompletionItemKind(kind) {
    switch (kind) {
        case CompletionKind.attribute:
        case CompletionKind.htmlAttribute:
        case CompletionKind.property:
            return lsp.CompletionItemKind.Property;
        case CompletionKind.component:
        case CompletionKind.element:
        case CompletionKind.key:
            return lsp.CompletionItemKind.Class;
        case CompletionKind.method:
            return lsp.CompletionItemKind.Method;
        case CompletionKind.pipe:
            return lsp.CompletionItemKind.Function;
        case CompletionKind.type:
            return lsp.CompletionItemKind.Interface;
        case CompletionKind.reference:
        case CompletionKind.variable:
            return lsp.CompletionItemKind.Variable;
        case CompletionKind.entity:
        default:
            return lsp.CompletionItemKind.Text;
    }
}
/**
 * Convert ts.CompletionEntry to LSP Completion Item.
 * @param entry completion entry
 * @param position position where completion is requested.
 */
function tsCompletionEntryToLspCompletionItem(entry, position) {
    var item = lsp.CompletionItem.create(entry.name);
    // Even though `entry.kind` is typed as ts.ScriptElementKind, it's
    // really Angular's CompletionKind. This is because ts.ScriptElementKind does
    // not sufficiently capture the HTML entities.
    // This is a limitation of being a tsserver plugin.
    var kind = entry.kind;
    item.kind = ngCompletionKindToLspCompletionItemKind(kind);
    item.detail = entry.kind;
    item.sortText = entry.sortText;
    item.textEdit = lsp.TextEdit.insert(position, entry.name);
    return item;
}
exports.tsCompletionEntryToLspCompletionItem = tsCompletionEntryToLspCompletionItem;
//# sourceMappingURL=completion.js.map