"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts = __importStar(require("typescript/lib/tsserverlibrary")); // used as value
var lsp = __importStar(require("vscode-languageserver"));
var utils_1 = require("./utils");
/**
 * Convert ts.DiagnosticCategory to lsp.DiagnosticSeverity
 * @param category diagnostic category
 */
function tsDiagnosticCategoryToLspDiagnosticSeverity(category) {
    switch (category) {
        case ts.DiagnosticCategory.Warning:
            return lsp.DiagnosticSeverity.Warning;
        case ts.DiagnosticCategory.Error:
            return lsp.DiagnosticSeverity.Error;
        case ts.DiagnosticCategory.Suggestion:
            return lsp.DiagnosticSeverity.Hint;
        case ts.DiagnosticCategory.Message:
        default:
            return lsp.DiagnosticSeverity.Information;
    }
}
/**
 * Convert ts.Diagnostic to lsp.Diagnostic
 * @param tsDiag TS diagnostic
 * @param scriptInfo Used to compute proper offset.
 */
function tsDiagnosticToLspDiagnostic(tsDiag, scriptInfo) {
    var textSpan = {
        start: tsDiag.start || 0,
        length: tsDiag.length || 0,
    };
    return lsp.Diagnostic.create(utils_1.tsTextSpanToLspRange(scriptInfo, textSpan), ts.flattenDiagnosticMessageText(tsDiag.messageText, '\n'), tsDiagnosticCategoryToLspDiagnosticSeverity(tsDiag.category), tsDiag.code, tsDiag.source);
}
exports.tsDiagnosticToLspDiagnostic = tsDiagnosticToLspDiagnostic;
//# sourceMappingURL=diagnostic.js.map