import * as ts from 'typescript/lib/tsserverlibrary';
import * as lsp from 'vscode-languageserver';
/**
 * `ProjectService` is a singleton service for the entire lifespan of the
 * language server. This specific implementation is a very thin wrapper
 * around TypeScript's `ProjectService`. On creation, it spins up tsserver and
 * loads `@angular/language-service` as a tsserver plugin.
 * Care must be taken to load `@angular/language-service` from the project or
 * workspace. This extension shoud not bundle its own
 * `@angular/language-service`, not even as a backup. This is to ensure that
 * the extension becomes a no-op when non-Angular TS projects are opened.
 * `ProjectService` is used to manage both TS document as well as HTML.
 * Using tsserver to handle non-TS files is fine as long as the ScriptKind is
 * configured correctly and `getSourceFile()` is never called on non-TS files.
 */
export declare class ProjectService {
    private readonly serverHost;
    private readonly logger;
    private readonly connection;
    readonly tsProjSvc: ts.server.ProjectService;
    constructor(serverHost: ts.server.ServerHost, logger: ts.server.Logger, connection: lsp.IConnection, options: Map<string, string>);
    /**
     * Return the default project for the specified `scriptInfo` if it is already
     * a configured project. If not, attempt to find a relevant config file and
     * make that project its default. This method is to ensure HTML files always
     * belong to a configured project instead of the default behavior of being in
     * an inferred project.
     * @param scriptInfo
     */
    getDefaultProjectForScriptInfo(scriptInfo: ts.server.ScriptInfo): ts.server.Project | undefined;
    /**
     * An event handler that gets invoked whenever the program changes and
     * TS ProjectService sends `ProjectUpdatedInBackgroundEvent`. This particular
     * event is used to trigger diagnostic checks.
     * @param event
     */
    private handleProjectServiceEvent;
}
