"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var ts = __importStar(require("typescript/lib/tsserverlibrary")); // used as value
/**
 * `ServerHost` is a wrapper around `ts.sys` for the Node system. In Node, all
 * optional methods of `ts.System` are implemented.
 * See https://github.com/microsoft/TypeScript/blob/ec39d412876d0dcf704fc886d5036cb625220d2f/src/compiler/sys.ts#L716
 */
var ServerHost = /** @class */ (function () {
    function ServerHost() {
        this.args = ts.sys.args;
        this.newLine = ts.sys.newLine;
        this.useCaseSensitiveFileNames = ts.sys.useCaseSensitiveFileNames;
    }
    ServerHost.prototype.write = function (s) {
        ts.sys.write(s);
    };
    ServerHost.prototype.writeOutputIsTTY = function () {
        return ts.sys.writeOutputIsTTY();
    };
    ServerHost.prototype.readFile = function (path, encoding) {
        return ts.sys.readFile(path, encoding);
    };
    ServerHost.prototype.getFileSize = function (path) {
        return ts.sys.getFileSize(path);
    };
    ServerHost.prototype.writeFile = function (path, data, writeByteOrderMark) {
        return ts.sys.writeFile(path, data, writeByteOrderMark);
    };
    /**
     * @pollingInterval - this parameter is used in polling-based watchers and
     * ignored in watchers that use native OS file watching
     */
    ServerHost.prototype.watchFile = function (path, callback, pollingInterval) {
        return ts.sys.watchFile(path, callback, pollingInterval);
    };
    ServerHost.prototype.watchDirectory = function (path, callback, recursive) {
        return ts.sys.watchDirectory(path, callback, recursive);
    };
    ServerHost.prototype.resolvePath = function (path) {
        return ts.sys.resolvePath(path);
    };
    ServerHost.prototype.fileExists = function (path) {
        return ts.sys.fileExists(path);
    };
    ServerHost.prototype.directoryExists = function (path) {
        return ts.sys.directoryExists(path);
    };
    ServerHost.prototype.createDirectory = function (path) {
        return ts.sys.createDirectory(path);
    };
    ServerHost.prototype.getExecutingFilePath = function () {
        return ts.sys.getExecutingFilePath();
    };
    ServerHost.prototype.getCurrentDirectory = function () {
        return ts.sys.getCurrentDirectory();
    };
    ServerHost.prototype.getDirectories = function (path) {
        return ts.sys.getDirectories(path);
    };
    ServerHost.prototype.readDirectory = function (path, extensions, exclude, include, depth) {
        return ts.sys.readDirectory(path, extensions, exclude, include, depth);
    };
    ServerHost.prototype.getModifiedTime = function (path) {
        return ts.sys.getModifiedTime(path);
    };
    ServerHost.prototype.setModifiedTime = function (path, time) {
        return ts.sys.setModifiedTime(path, time);
    };
    ServerHost.prototype.deleteFile = function (path) {
        return ts.sys.deleteFile(path);
    };
    /**
     * A good implementation is node.js' `crypto.createHash`.
     * (https://nodejs.org/api/crypto.html#crypto_crypto_createhash_algorithm)
     */
    ServerHost.prototype.createHash = function (data) {
        return ts.sys.createHash(data);
    };
    /**
     * This must be cryptographically secure. Only implement this method using
     * `crypto.createHash("sha256")`.
     */
    ServerHost.prototype.createSHA256Hash = function (data) {
        return ts.sys.createSHA256Hash(data);
    };
    ServerHost.prototype.getMemoryUsage = function () {
        return ts.sys.getMemoryUsage();
    };
    ServerHost.prototype.exit = function (exitCode) {
        return ts.sys.exit(exitCode);
    };
    ServerHost.prototype.realpath = function (path) {
        return ts.sys.realpath(path);
    };
    ServerHost.prototype.setTimeout = function (callback, ms) {
        var _a;
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        return (_a = ts.sys).setTimeout.apply(_a, [callback, ms].concat(args));
    };
    ServerHost.prototype.clearTimeout = function (timeoutId) {
        return ts.sys.clearTimeout(timeoutId);
    };
    ServerHost.prototype.clearScreen = function () {
        return ts.sys.clearScreen();
    };
    ServerHost.prototype.base64decode = function (input) {
        return ts.sys.base64decode(input);
    };
    ServerHost.prototype.base64encode = function (input) {
        return ts.sys.base64encode(input);
    };
    ServerHost.prototype.setImmediate = function (callback) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        return setImmediate.apply(void 0, [callback].concat(args));
    };
    ServerHost.prototype.clearImmediate = function (timeoutId) {
        return clearImmediate(timeoutId);
    };
    ServerHost.prototype.require = function (initialPath, moduleName) {
        try {
            var modulePath = require.resolve(moduleName, {
                paths: [initialPath],
            });
            return {
                module: require(modulePath),
                error: undefined,
            };
        }
        catch (e) {
            return {
                module: undefined,
                error: e,
            };
        }
    };
    return ServerHost;
}());
exports.ServerHost = ServerHost;
//# sourceMappingURL=server_host.js.map