import * as lsp from 'vscode-languageserver';
import * as ts from 'typescript/lib/tsserverlibrary';
/**
 * Extract the file path from the specified `uri`.
 * @param uri
 */
export declare function uriToFilePath(uri: string): string;
/**
 * Converts the specified `filePath` to a proper URI.
 * @param filePath
 */
export declare function filePathToUri(filePath: string): string;
/**
 * Convert ts.TextSpan to lsp.TextSpan. TypeScript keeps track of offset using
 * 1-based index whereas LSP uses 0-based index.
 * @param scriptInfo Used to determine the offsets.
 * @param textSpan
 */
export declare function tsTextSpanToLspRange(scriptInfo: ts.server.ScriptInfo, textSpan: ts.TextSpan): lsp.Range;
/**
 * Convert lsp.Position to the absolute offset in the file. LSP keeps track of
 * offset using 0-based index whereas TypeScript uses 1-based index.
 * @param scriptInfo Used to determine the offsets.
 * @param position
 */
export declare function lspPositionToTsPosition(scriptInfo: ts.server.ScriptInfo, position: lsp.Position): number;
/**
 * Convert lsp.Range which is made up of `start` and `end` positions to
 * TypeScript's absolute offsets.
 * @param scriptInfo Used to determine the offsets.
 * @param range
 */
export declare function lspRangeToTsPositions(scriptInfo: ts.server.ScriptInfo, range: lsp.Range): [number, number];
