"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
var lsp = __importStar(require("vscode-languageserver"));
var vscode_uri_1 = require("vscode-uri");
var Scheme;
(function (Scheme) {
    Scheme["File"] = "file";
})(Scheme || (Scheme = {}));
/**
 * Extract the file path from the specified `uri`.
 * @param uri
 */
function uriToFilePath(uri) {
    // Note: uri.path is different from uri.fsPath
    // See https://github.com/microsoft/vscode-uri/blob/413805221cc6ed167186ab3103d3248d6f7161f2/src/index.ts#L622-L645
    var _a = vscode_uri_1.URI.parse(uri), scheme = _a.scheme, fsPath = _a.fsPath;
    if (scheme !== Scheme.File) {
        return '';
    }
    return fsPath;
}
exports.uriToFilePath = uriToFilePath;
/**
 * Converts the specified `filePath` to a proper URI.
 * @param filePath
 */
function filePathToUri(filePath) {
    return vscode_uri_1.URI
        .from({
        scheme: Scheme.File,
        path: filePath,
    })
        .toString();
}
exports.filePathToUri = filePathToUri;
/**
 * Convert ts.TextSpan to lsp.TextSpan. TypeScript keeps track of offset using
 * 1-based index whereas LSP uses 0-based index.
 * @param scriptInfo Used to determine the offsets.
 * @param textSpan
 */
function tsTextSpanToLspRange(scriptInfo, textSpan) {
    var start = scriptInfo.positionToLineOffset(textSpan.start);
    var end = scriptInfo.positionToLineOffset(textSpan.start + textSpan.length);
    // ScriptInfo (TS) is 1-based, LSP is 0-based.
    return lsp.Range.create(start.line - 1, start.offset - 1, end.line - 1, end.offset - 1);
}
exports.tsTextSpanToLspRange = tsTextSpanToLspRange;
/**
 * Convert lsp.Position to the absolute offset in the file. LSP keeps track of
 * offset using 0-based index whereas TypeScript uses 1-based index.
 * @param scriptInfo Used to determine the offsets.
 * @param position
 */
function lspPositionToTsPosition(scriptInfo, position) {
    var line = position.line, character = position.character;
    // ScriptInfo (TS) is 1-based, LSP is 0-based.
    return scriptInfo.lineOffsetToPosition(line + 1, character + 1);
}
exports.lspPositionToTsPosition = lspPositionToTsPosition;
/**
 * Convert lsp.Range which is made up of `start` and `end` positions to
 * TypeScript's absolute offsets.
 * @param scriptInfo Used to determine the offsets.
 * @param range
 */
function lspRangeToTsPositions(scriptInfo, range) {
    var start = lspPositionToTsPosition(scriptInfo, range.start);
    var end = lspPositionToTsPosition(scriptInfo, range.end);
    return [start, end];
}
exports.lspRangeToTsPositions = lspRangeToTsPositions;
//# sourceMappingURL=utils.js.map